## @class
# The (Ex)ternal (Pe)rformance (Da)ta (P)rovider requester. This requester
# requires an external program to handle the actual requests and return it
# to print "variable=value" lines on STDOUT.
package Requester::expedap;

use strict;
use Requester::SlowRequester;

our(@ISA);
@ISA = qw(Requester::SlowRequester);

my $expedap = new Requester::expedap();

## @cmethod new()
#
sub new
{
   my( $this, @args ) = @_;
   my $class = ref($this) || $this;
   my $self = {};
   bless( $self, $class );
   $self->register("expedap");	 
   return( $self );
}

## @method $ startup( @args )
# Starts the expedap external command (and extends the path so that it contains
# the expedap directory). Passes arguments and return values through to/from the
# superclass method.
sub startup {
  my ($self, @args) = @_;
	my $sysPath = $ENV{PATH};
	Platform::mergepath( $BigSister::common::fs{"adm"}."/expedap", 0);
	print "search path is: $ENV{PATH}\n" if ($BigSister::common::dl > 4);
	my $fh = $self -> SUPER::startup(@args);
	$ENV{PATH} = $sysPath;
	print "search path is reset to: $ENV{PATH}\n" if ($BigSister::common::dl > 4);
	return $fh;
}

## @method hashref getinfo( arrayref vars, arrayref indexes )
# Just calls walk with parameter list extended by an empty walks list.
sub getinfo {
    my( $self, $vars, $indexes ) = @_;
    return( $self->getwalk( $vars, $indexes, [] ) );
}

## @method hashref getwalk( arrayref vars, arrayref indexes, arrayref walks )
# Mainly determines the command from the variable list and the tester environment and then
# uses the superclass' getinfo() method.
sub getwalk {
    my( $self, $vars, $indexes, $walks ) = @_;

    return( undef ) unless( ($vars && @$vars) || ($walks && @$walks) );
	my $cmd = ($vars && @$vars) ? $vars->[0] : $walks->[0];
	$cmd =~ s/\..*$//;
	
	# calculate arguments
	my $args = $self -> {"env"} -> {"args"};
	
	# if argument_list is set, then positional parameters are used, else all
	# parameters in $args are passed als named parameters
	my $argument_list = $self -> {"env"} -> expand_values( "argument_list" );
	print "argument_list: @$argument_list\n" if ($BigSister::common::dl > 4);
	if ( scalar @$argument_list > 0) {
	  #positional arguments
		foreach my $key ( @$argument_list ) {
			 $cmd .= " ".$args -> { $key }."" if ( exists $args -> { $key } );
		}
	} else {
		foreach my $key ( keys %$args ) {
			$cmd .= " $key=".$args -> { $key }."";
		}
	}
	
	$self -> command( $cmd );
	print "Expedap command is: $cmd\n" if ($BigSister::common::dl > 4);
	
	# the slowrequester object
	my $srq = $self -> SUPER::getinfo($vars, $indexes);
	
	# pass the baseId to the slowrequester
	$srq->{"Expedap::baseId"} = $self->{"env"}->expand_value( "id" );
	$srq->{"Expedap::uniqprefix"} = $self->{"env"}->expand_value( "unique_prefix" );
	print "getinfo baseId: ".$srq->{"Expedap::baseId"}."\n" if ($BigSister::common::dl > 4);
	
	return $srq;
}


## @method void do( $srq )
# Accepts lines of the form
#
#    variable=value
#
# as written to the STDOUT of the expedap external program.
sub do {
	my ($self, $srq) = @_;
	return unless (my $line = $srq -> readln());
	
	
	while ( $line ) {
		chomp($line);

		if ( $line =~ /END/ ) {
			$srq -> status(3, "finished OK");
			return;
		}
		 
		if ( $line =~ /(.*)\.(\d+)\s*=\s*(.*)/ ) {
			my( $var, $id, $value ) = ($1,$2,$3);
			print "test result: $line\n" if ($BigSister::common::dl > 4);
			
			my $baseId = $srq->{"Expedap::baseId"};
			print "do baseId: $baseId\n" if ($BigSister::common::dl > 6);
			$id = $baseId + $id if( $baseId );

			my $prefix = $srq->{"Expedap::uniqprefix"};
			if( $prefix ) {
			    $var =~ s#\.#.$prefix.#;
			}
			
			my $info = $srq -> {"Test.info"};
			$info -> { $id } -> { $var } = $value;
			print "info -> { $id } -> { $var } = $value\n" if ($BigSister::common::dl > 6);
		}

		$line = $srq -> readln();
	}	
}

1;
